package cis690.insight.debug;

import java.util.Observable;
import java.util.Vector;

import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IDebugEventSetListener;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IThread;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * 
 * @version May 21, 2004
 * @author charlie
 */
public class DebugModel extends Observable
	implements IDebugModel, IDebugEventSetListener
{
	private Vector m_objects;

	public DebugModel()
	{
		m_objects = new DOVector();
		IDebugTarget target = DebugUtil.getDebugTarget();
		
		if (target != null)
		{
			beginSession(target);
			notifyObservers();
		}
		DebugPlugin.getDefault().addDebugEventListener(this);
	}

	private void endSession()
	{
		m_objects.removeAllElements();
		System.out.println("ending session");
		setChanged();
	}

	private void addThread(IJavaThread t)
	{
		IDebugObject obj = new DebugObject(this, t);
		m_objects.add(obj);
		setChanged();
	}
	
	private void removeThread(IJavaThread t)
	{
		if (m_objects.contains(t))
		{
			m_objects.remove(t);
			setChanged();
		}
		assert(!m_objects.contains(t));
	}

	private void beginSession(IDebugTarget target)
	{
		endSession(); // ensure that any previous session has ended
		
		try
		{
			if (target.hasThreads())
			{
				IThread[] threads = target.getThreads();
				IJavaThread t;
				for (int i=0; i<threads.length; ++i)
				{
					t = (IJavaThread)threads[i].getAdapter(IJavaThread.class);
					if (!isSystemThread(t))
					{
						addThread(t);
					}
				}
			}
		}
		catch (Exception e) {e.printStackTrace();}
	}

	public Vector getObjects()
	{
		return m_objects;
	}

	public void open(IDebugObject parent, IVariable child)
	{
		IDebugObject obj = new DebugObject(this, child);
		
		if (!m_objects.contains(obj))
		{
			m_objects.add(obj);	
		}
		else
		{
			obj = (IDebugObject)m_objects.get(m_objects.indexOf(obj));	
		}
		
		obj.addParent(parent);
		parent.addChild(obj);
		
		setChanged();
		notifyObservers();
	}

	public void close(IDebugObject parent, IVariable child)
	{
		assert m_objects.contains(parent);
		assert m_objects.contains(child);
		
		IDebugObject obj = (IDebugObject)m_objects.get(m_objects.indexOf(child));
		obj.remParent(parent);
		parent.remChild(obj);
		if (obj.getParents().isEmpty() && !obj.isThread())
		{
			m_objects.remove(obj);
			assert(!m_objects.contains(child));
		}
		setChanged();
		notifyObservers();
	}

	public void handleDebugEvents(DebugEvent[] events)
	{
		for (int i=0; i<events.length; ++i)
		{
			if (events[i].getSource() instanceof IDebugTarget)
			{
				switch (events[i].getKind())
				{
					case DebugEvent.CREATE:
						beginSession((IDebugTarget)events[i].getSource());
						break;
					case DebugEvent.TERMINATE:
						endSession();
						break;
					default:
						break;
				}
			}
			else if (events[i].getSource() instanceof IJavaThread &&
					 !isSystemThread(events[i].getSource()))
			{
				switch (events[i].getKind())
				{
					case DebugEvent.CREATE:
						addThread((IJavaThread)events[i].getSource());
						break;
					case DebugEvent.SUSPEND:
						getModelObj(events[i].getSource()).setSuspended(true);
						setChanged();
						break;
					case DebugEvent.RESUME:
						getModelObj(events[i].getSource()).setSuspended(false);
						setChanged();
						break;
					case DebugEvent.TERMINATE:
						removeThread((IJavaThread)events[i].getSource());
						break;
					default:
						break;
				}
			}
		}
		
		for (int i=0; i<m_objects.size(); ++i)
		{
			((Observable)m_objects.get(i)).notifyObservers();
		}
		notifyObservers();
	}
	
	private boolean isSystemThread(Object obj)
	{
		try
		{
			if (obj instanceof IJavaThread &&
				((IJavaThread)obj).isSystemThread())
			{
				return true;	
			}
		}
		catch (DebugException e) { e.printStackTrace(); }
		
		return false;
	}
	
	private IDebugObject getModelObj(Object obj)
	{	
		for (int i=0; i<m_objects.size(); ++i)
		{
			if (m_objects.get(i).equals(obj))
			{
				return (IDebugObject)m_objects.get(i);	
			}	
		}
		assert(false);
		return null;
	}
	

}
