package cis690.insight.debug;

import java.util.Observable;
import java.util.Vector;

import org.eclipse.debug.core.model.IVariable;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jdt.debug.core.IJavaThread;

/**
 * 
 * @version May 21, 2004
 * @author charlie
 */
public class DebugObject extends Observable
	implements IDebugObject
{
	// the big guy that keeps track of all us objects
	// calls to open and close are passed up to the model
	private IDebugModel m_model;
	
	// family hierarchy
	private Vector m_parents;
	private Vector m_children;
	
	// state information
	private boolean m_isThread;
	private boolean m_isSuspended;
	private String m_name;
	private IVariable[] m_vars;
	private Point m_xy;
	private boolean m_shade;
	
	// the underlying object
	private Object m_ref;
	
	private DebugObject(IDebugModel model, Object ref, IVariable[] vars, String name)
	{
		m_ref = ref;
		m_model = model;
		m_vars = vars;
		m_name = name;
		m_parents = new DOVector();
		m_children = new DOVector();	
		m_isThread = false;
		m_isSuspended = false;
		m_xy = new Point(6,6);
		m_shade = false;
	}
	
	// because we are not attaching a DebugPlugin listen, updates will have
	// to be explicity sent from thread objects to child objects
	public DebugObject(IDebugModel model, IVariable var)
	{
		this(model, var, DebugUtil.getVarChildren(var), DebugUtil.getVarName(var));
	}
	
	public DebugObject(IDebugModel model, IJavaThread t)
	{
		this(model, t, DebugUtil.getVisibleVars(t), DebugUtil.getThreadName(t));
		m_isThread = true;
	}
	
	public IDebugModel getDebugModel()
	{
		return m_model;	
	}
	
	public boolean isThread()
	{
		return m_isThread;	
	}
	
	public Vector getParents()
	{
		return m_parents;	
	}
	
	public void addParent(IDebugObject parent)
	{
		assert(!m_parents.contains(parent));
		m_parents.add(parent);
	}
	
	public void remParent(IDebugObject parent)
	{
		assert(m_parents.contains(parent));
		m_parents.remove(parent);
	}
	
	public Vector getChildren()
	{
		return m_children;
	}
	
	public void addChild(IDebugObject child)
	{
		assert(!m_children.contains(child));
		m_children.add(child);
	}
	
	public void remChild(IDebugObject child)
	{
		assert(m_children.contains(child));
		m_children.remove(child);
	}
	
	public Object getObj()
	{
		return m_ref;	
	}
	
	public String getName()
	{
		return m_name;	
	}
	
	public IVariable[] getVars()
	{
		// since a thread's list of visible variables is always changing, we
		// want to grab a fresh one each time
		if (m_ref instanceof IJavaThread)
		{
			return DebugUtil.getVisibleVars((IJavaThread)m_ref);	
		}
		else if (m_ref instanceof IVariable)
		{
			return m_vars;
		}
		return new IVariable[0];
	}
	
	public Point getXY()
	{
		return m_xy;	
	}
	
	public void setXY(Point pt)
	{
		m_xy = pt;
		setChanged();
	}
	
	public void setShade(boolean shadeMode)
	{
		m_shade = shadeMode;
		
		if (shadeMode)
		{
			DebugObject[] childs = new DebugObject[m_children.size()];
			m_children.toArray(childs);
			for (int i=0; i<childs.length; ++i)
			{
				close((IVariable)childs[i].m_ref);	
			}
			assert(m_children.size() == 0);
		}
		setChanged();
	}
	
	public boolean isShade()
	{
		return m_shade;	
	}
	
	public boolean isSuspended()
	{		
		return getSuspendedParentThread() != null;
	}
	
	private IJavaThread getSuspendedParentThread()
	{
		if (isThread() && ((IJavaThread)m_ref).isSuspended())
		{
			return (IJavaThread)m_ref;
		}
		
		final Vector parents = getParents();
		for (int i=0; i<parents.size(); ++i)
		{
			IJavaThread ijt = ((DebugObject)parents.get(i)).getSuspendedParentThread();
			if (ijt != null)
			{
				return ijt;
			}
		}
		return null;
	}
	
	public void setSuspended(boolean suspended)
	{
		m_isSuspended = suspended;
		setChanged();
		for (int i=0; i<m_children.size(); ++i)
		{
			if (!((DebugObject)m_children.get(i)).isThread())
			{
				((DebugObject)m_children.get(i)).setSuspended(suspended);
			}
		}
	}
	
	public void notifyObservers()
	{
		for (int i=0; i<m_children.size(); ++i)
		{
			if (!((DebugObject)m_children.get(i)).isThread())
			{
				((DebugObject)m_children.get(i)).notifyObservers();
			}
		}
		super.notifyObservers();
	}
	
	public void close(IVariable var)
	{
		m_model.close(this, var);
	}
	
	public void open(IVariable var)
	{
		m_model.open(this, var);
	}
	
	public boolean equals(Object obj)
	{
		return DebugUtil.symanticEquality(this, obj);
	}

}
