package cis690.insight.figures;


import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;

import cis690.insight.Colors;
import cis690.insight.Fonts;
import cis690.insight.debug.DebugUtil;

/**
 * 
 * @version Apr 23, 2004
 * @author charlie
 */
public class ObjFig extends RectangleFigure
{
	private HeaderFigure m_header;
	private ContentFig m_content;
	private ObjFigListener m_listen;

	/** use this one for the figure from a thread */
	public ObjFig(String name, boolean isThread)
	{	
		setBorder(new LineBorder(Colors.BORDER, 1));
		ToolbarLayout lMan = new ToolbarLayout(false);
		lMan.setStretchMinorAxis(true);
		lMan.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
		lMan.setSpacing(0);
		setLayoutManager(lMan);
		
		m_header = new HeaderFigure(name, isThread);
		m_content = new ContentFig();
		m_listen = null;
		
		add(m_header);
		add(m_content);
	}
	
	public void setObjListener(ObjFigListener ofl)
	{
		m_listen = ofl;
	}
	
	public void setVars(IVariable[] vars)
	{
		m_content.setVars(vars);
	}
	
	public int getHeaderHeight()
	{
		return m_header.getPreferredSize().height;
	}
	
	public ConnectionAnchor getAnchor()
	{
		return m_header.getAnchor();
	}
	
	public ConnectionAnchor getSourceAnchor(Object var)
	{
		return m_content.anchorFor(var);
	}
	
	private class HeaderFigure extends Panel
	{
		private Label i_title;
		private ImageFigure i_icon;
		
		public HeaderFigure(String name, boolean isThread)
		{
			BorderLayout layout = new BorderLayout();
			layout.setHorizontalSpacing(6);
			setLayoutManager(layout);
			setBackgroundColor(Colors.FIG_HEADER);
			setForegroundColor(Colors.FIG_HTEXT);
			setOpaque(true);
						
			ImageData imgData = null;
			if (isThread)
			{
			    imgData = ImageDescriptor.createFromFile(getClass(), "thread.png").getImageData();
			}
			else
			{
			    imgData = ImageDescriptor.createFromFile(getClass(), "bullet_ball_glass_blue.png").getImageData();
			}
			
			Image img = new Image(Display.getDefault(), imgData);
			i_icon = new ImageFigure(img);
			
			i_title = new Label(name);
			i_title.setFont(Fonts.HEADER_FONT);
			i_title.setTextAlignment(PositionConstants.LEFT);
			i_title.setLabelAlignment(PositionConstants.LEFT);
			
			add(i_icon, BorderLayout.LEFT);
			add(i_title, BorderLayout.CENTER);
			add(new ExpandButton(), BorderLayout.RIGHT);
		}

		public Insets getInsets()
		{
			return new Insets(3,3,3,3);
		}
		
		public ConnectionAnchor getAnchor()
		{
			return new ChopboxAnchor(i_icon);
		}
	}
	
	private class ExpandButton extends Button implements ActionListener
	{
		public ExpandButton()
		{
			super("...");
			setFont(Fonts.BUTTON_FONT);
			setForegroundColor(Colors.FIG_BTEXT);
			setPreferredSize(14,8);
			addActionListener(this);
		}
		
		public void actionPerformed(ActionEvent evt)
		{
			m_listen.toggleShade();
		}
	}

	private static Object findEquivKey(Hashtable hash, Object key)
	{
		Enumeration keys = hash.keys();
		while (keys.hasMoreElements())
		{
			final Object curKey = keys.nextElement();
			if (DebugUtil.getObjectID(key) == DebugUtil.getObjectID(curKey))
			{
				return curKey;
			}
		}
		return key;
	}
	
	private class ContentFig extends Figure implements ActionListener
	{
		private Hashtable i_varHash; // of IVariable --> VarFig
		
		public ContentFig()
		{
			setBackgroundColor(Colors.FIG_BACK);
			setForegroundColor(Colors.FIG_BTEXT);
			
			ToolbarLayout lMan = new ToolbarLayout(false);
			lMan.setStretchMinorAxis(true);
			lMan.setSpacing(0);
			setLayoutManager(lMan);
			
			i_varHash = new Hashtable();
		}
		
		public ConnectionAnchor anchorFor(Object var)
		{
			return ((VarFig)i_varHash.get(findEquivKey(i_varHash, var))).getAnchorOut();	
		}
		
		public void setVars(IVariable[] vars)
		{
			IVariable[] known = new IVariable[i_varHash.size()];
			i_varHash.keySet().toArray(known);
			
			// add any new variable figures
			for (int i=0; i<vars.length; ++i)
			{
				if (!arrayContains(known, vars[i]))
				{
					String type = DebugUtil.getVarTypeName(vars[i]);
					String name = DebugUtil.getVarName(vars[i]);
					VarFig vf = new VarFig(type, name);
					add(vf);
					i_varHash.put(vars[i], vf);
					vf.addExpandListener(this);
				}	
			}
			
			// remove any stale variable figures
			for (int i=0; i<known.length; ++i)
			{
				if (!arrayContains(vars, known[i]))
				{
					VarFig vf = (VarFig)i_varHash.get(known[i]);
					remove(vf);
					i_varHash.remove(known[i]);
				}
			}
			
			// refresh all the current variables
			known = new IVariable[i_varHash.size()];
			i_varHash.keySet().toArray(known);
			for (int i=0; i<known.length; ++i)
			{
				((VarFig)i_varHash.get(known[i])).refresh(known[i]);
			}
			
			revalidate();
		}

		public void actionPerformed(ActionEvent event)
		{
			// since the VarFig doesn't actually know which variable it
			// represents, we have to do a reverse lookup in the hash
			IFigure source = (IFigure)event.getSource();
			IVariable[] known = new IVariable[i_varHash.size()];
			i_varHash.keySet().toArray(known);
			for (int i=0; i<known.length; ++i)
			{
				VarFig f = (VarFig)i_varHash.get(known[i]);
				if (f.getChildren().contains(source))
				{
					m_listen.toggleVar(known[i]);
					return;
				}
			}
			assert(false);
		}
	}

	private static boolean arrayContains(IVariable[] arr, IVariable val)
	{
		for (int i=0; i<arr.length; ++i)
		{
			try
			{
				if (arr[i].getName().equals(val.getName()))
				{
					return true;	
				}
			}
			catch (DebugException e)
			{
				e.printStackTrace();	
			}
		}
		return false;
	}

}
