package cis690.insight.figures;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.draw2d.ActionListener;
import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Button;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Panel;
import org.eclipse.jdt.debug.core.IJavaPrimitiveValue;

import cis690.insight.Colors;
import cis690.insight.Fonts;
import cis690.insight.debug.DebugUtil;


/**
 * 
 * @version May 17, 2004
 * @author charlie
 */
public class VarFig extends Panel
{
	private Label m_desc;
	private Label m_value;
	private ExpandButton m_expand;
	
	public VarFig(String type, String name)
	{
		setLayoutManager(new BorderLayout());
		
		
		m_value = new InnerLabel(" ");
		m_desc = new InnerLabel("("+type+") "+name+": ");
		m_expand = new ExpandButton();
		m_expand.setVisible(false);
		
		add(m_desc, BorderLayout.LEFT);
		add(m_value, BorderLayout.CENTER);
		add(m_expand, BorderLayout.RIGHT);
	}
	
	public ConnectionAnchor getAnchorOut()
	{
		return new ChopboxAnchor(m_expand);
	}
	
	public void addExpandListener(ActionListener al)
	{
		m_expand.addActionListener(al);	
	}
	
	public void refresh(IVariable var)
	{
		m_expand.setVisible(canExpand(var));
		String str = DebugUtil.getVarValueStr(var).trim();
		if (DebugUtil.isStr(var))
		{
			str = "\"" + str + "\"";
		}
		if (!str.equals(m_value.getText()))
		{
			m_value.setForegroundColor(Colors.FIG_VAL_CH);
		}
		else
		{
			m_value.setForegroundColor(Colors.FIG_VAL);
		}
		m_value.setText(str);
	}
	
	private static boolean canExpand(IVariable var)
	{
		try
		{
			return !(var.getValue() instanceof IJavaPrimitiveValue) &&
				   !DebugUtil.isStr(var) &&
				   !(var.getValue().getValueString().equals("null"));
		}
		catch (DebugException e) { e.printStackTrace(); }
		return false;
	}
	
	private class ExpandButton extends Button
	{
		public ExpandButton()
		{
			super("...");
			setFont(Fonts.BUTTON_FONT);
			setForegroundColor(Colors.FIG_BTEXT);
			setPreferredSize(14,8);
		}
	}

}
